package de.gruschke.dasync;

import java.io.PrintStream;

/**
 * Incrementally updates the Directory Server.
 * {@link de.gruschke.dasync.MapComparer#compare(Map, Map) super.compare(Map,Map)}
 * compares two Maps. Methods implemented here invoke methods from
 * {@link de.gruschke.dasync.UpdateDSDataRow} for each data object that needs its
 * directory entry to be updated.
 *
 * @author Boris Gruschke
*/

public class UpdateDSMapComparer
    extends MapComparer {

    public UpdateDSMapComparer() {}

    public UpdateDSMapComparer(boolean verbose) {
	this.verbose = verbose;
    }

    /**
      * Invoke {@link de.gruschke.dasync.UpdateDSDataRow#addToDS()
      * UpdateDSDataRow.addToDS()}
      */
    protected void inOneOnly(Object key, Object value) {
	if (verbose) {
	    out.println("*** Adding to DS: Key=" + key +
			"; Entry="+ value);
	}
	((UpdateDSDataRow)value).addToDS();
    }

    /**
      * Invoke {@link de.gruschke.dasync.UpdateDSDataRow#removeFromDS()
      * UpdateDSDataRow.removeFromDS()}
      */
    protected void inTwoOnly(Object key, Object value) {
	if (verbose) {
	    out.println("*** Removing from DS: Key=" + key +
			"; Entry="+ value);
	}
	((UpdateDSDataRow)value).removeFromDS();
    }

    /**
      * Invoke {@link de.gruschke.dasync.UpdateDSDataRow#modifyInDS(Object)
      * UpdateDSDataRow.modifyInDS(...)}
      */
    protected void oneAndTwoDiffer(Object key,
				   Object valueOne,
				   Object valueTwo) {
	if (verbose) {
	    out.println("*** Updating DS: Key=" + key +
			"; EntryDS=" + valueTwo + "; EntryAS=" + valueOne);
	}
	((UpdateDSDataRow)valueTwo).modifyInDS(valueOne);
    }

    protected void oneAndTwoEqual(Object key, Object value) {
    }

    private boolean verbose = false;
    private PrintStream out = System.out;
}
