package de.gruschke.dasync;

import javax.naming.directory.DirContext;

import javax.naming.Context;
import javax.naming.InvalidNameException;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttributes;


public class Utils {

    public static DirContext setupDirectories(DirContext initialCtx,
					String dirContextName,
					String suffix)
	throws InvalidNameException,
	NamingException {

	try {
	    return (DirContext)initialCtx.lookup(dirContextName);
	}
	catch(NameNotFoundException oops) {

	    if (suffix != null && suffix.equals(dirContextName)) {
		Attributes attrs = new BasicAttributes(objectclass, "top");
		return initialCtx.createSubcontext(dirContextName, attrs);
	    }

	    int firstRdnSep = dirContextName.indexOf(',');

	    if (firstRdnSep == -1) {
		return createSubcontext(initialCtx,
					dirContextName);
	    }

	    String firstContextName = dirContextName.substring(0,firstRdnSep);
	    String remainingContextName = dirContextName.substring(firstRdnSep+1);
	    
	    DirContext parentContext = setupDirectories(initialCtx,
							remainingContextName,
							suffix);

	    return createSubcontext(parentContext,
				    firstContextName);
	}
    }

    private static DirContext createSubcontext(DirContext context,
					       String rdn)
	throws NamingException,
	InvalidNameException {

	// Parse the Relative Distinguished Name (rdn)
	//
	rdn = rdn.trim();

	int eq = rdn.indexOf("=");
	if (eq == -1) {
	    throw new InvalidNameException("Relative Directory Name " + rdn
					   + " is invalid. The correct syntax for DNs is a comma (,) separated list of key=value pairs");
	}
	
	String key = rdn.substring(0,eq);
	String value = rdn.substring(eq + 1);

	// guess "good" attributes from DN
	//
	final String[] keyNames = { "dc", "o", "ou" };
	final int key_DC=0,
	    key_O=1,
	    key_OU=2,
	    key_ERROR=3; // always keyNames.length
	
	int keyId = 0;
	while (keyId<keyNames.length && !key.equals(keyNames[keyId])) {
	    keyId++;
	}

	Attributes attrs = null;

	switch (keyId) {
	case key_DC:
	    attrs = new BasicAttributes(objectclass,"dcObject");
	    break;
	case key_O:
	    attrs = new BasicAttributes(objectclass,"organization");
	    break;
	case key_OU:
	    attrs = new BasicAttributes(objectclass,"organizationalUnit");
	    break;
	case key_ERROR:
	    StringBuffer sb = new StringBuffer(keyNames[0]);
	    
	    for (int i=1;i<keyNames.length;i++) {
		sb.append(", " + keyNames[i]);
	    }

	    throw new InvalidNameException("Key " + key + " of RDN " + rdn +
					   " must be one of " + sb + ".");
	}

	attrs.put(key,value);

	return context.createSubcontext(rdn,attrs);
    }

    private final static String objectclass="objectclass";

}
